% Copyright (c) 2010 Iris Yan

% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.

% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%% This code is used to compare the proposed handover triggering condition 
%% estimation (HTCE) with other handover triggering methods, including 
%% fixed RSS based and hysteresis based methods.
clear all

%% Network parameters initialization.
R=150;R_fixed=130;R_hyst=120;tau_i=2;tau_o=2;P_b=0;

%% Velocities of mobile terminals from 1 km/h to 100 km/h are included.
for i=1:100
    v0(i)=i;
    v(i)=v0(i).*5/18;
    
%% Probability of connection breakdowns for the fixed RSS based method.
    if v(i)*tau_o<=(R-R_fixed)
        P_b_fixed(i)=0;
    elseif v(i)*tau_o>=(sqrt(R^2-R_fixed^2))
        P_b_fixed(i)=1;
    else P_b_fixed(i)=1-1/pi*(acos(2*R_fixed^2/R^2-(R^2-R_fixed^2-(v(i)*tau_o).^2).^2/2./(v(i)*tau_o).^2/R^2-1));
    end
    
%% Probability of connection breakdowns for the hysteresis based method.   
    if v(i)*tau_o<=R-R_hyst
        P_b_hyst(i)=0;
    elseif v(i)*tau_o>=sqrt(R^2-R_hyst^2)
        P_b_hyst(i)=1;
    else P_b_hyst(i)=1-1/pi*(acos(2*R_hyst^2/R^2-(R^2-R_hyst^2-(v(i)*tau_o).^2).^2/2./(v(i)*tau_o).^2/R^2-1));
    end
    
%% Probability of connection breakdowns for HTCE.
    P_b_HTCE(i)=P_b;
end

%% Plot the figure for probability of connection breakdowns.  
plot(v0,P_b_fixed,'-.b','LineWidth',2);
hold on
plot(v0,P_b_hyst,'--g','LineWidth',2);
hold on
plot(v0,P_b_HTCE,'-r','LineWidth',2);
hold on
legend('Fixed RSS based','Hysteresis based','HTCE');
grid  ('on')
xlabel('Velocity (km/h)')
ylabel('Connection Breakdown Probability')
