% Copyright (c) 2010 Iris Yan

% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.

% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.


%% This code is used to compare the proposed handover triggering condition 
%% estimation (HTCE) with other handover triggering methods, including 
%% fixed RSS based and hysteresis based methods.
clear all

%% Network parameters initialization.
R=150;
sigma0=4.3;
d_sample=30;
P_tx=20;
PL_ref=40;
tau_i=2;
tau_o=2;
beta=3.5;
v_htce=1;
R_fixed=130;
R_hyst=120;
RSS_fixed=P_tx-PL_ref-10*beta*log10(R_fixed);
RSS_hyst=P_tx-PL_ref-10*beta*log10(R_hyst);
RSS_HNE=P_tx-PL_ref-10*beta*log10(R);
v0=[3.6:2:100];
v=v0.*5/18;
x=v.*tau_o;

%% Four values of P_b are applied in the simulation: 0.02, 0.4, 0.6 and
%% 0.8.
for i=1:4
    if i==1
        Ph(i)=0.02;
    elseif i==2
        Ph(i)=0.4;
    elseif i==3
        Ph(i)=0.6;
    else Ph(i)=0.8;
    end
    
%% Velocities of mobile terminals from 3.6 km/h to 100 km/h in 2 km/h 
%% increments are simulated.
for ik = 1 :length(v)
    m=0;n=0;m1=0;m3=0;n1=0;n2=0;n3=0;m0=0;
    sample_interval(ik)=0.001.*v0(ik);
    N(ik)=floor(d_sample/sample_interval(ik));
    sigma(ik)=getsigma(sigma0,N(ik));
    errorratio(ik)=0.1./v(ik);
    distance_HTCE(i,ik) = 0;
    distance_HTCE_eff(i,ik) = 0;
    distance_ideal(ik)=0;
    distance_GHO(ik) = 0;
    distance_GHO_eff(ik) = 0;
    distance_fixed(ik) = 0;
    distance_fixed_eff(ik) = 0;
    num_break(i,ik) = 0;
    num_break_Min(ik) = 0;
    num_break_fixed(ik) = 0;
    esterror_ave(i,ik) = 0;
    esterror_sum(i,ik) = 0;
    wlantime(ik) = 0; 
    wuse(i,ik) = 0; 
    wuse_eff(i,ik) = 0;
    wuse_Min(ik) = 0;
    wuse_eff_Min(ik) = 0;
    wuse_fixed(ik) = 0;
    wuse_eff_fixed(ik) = 0;
    wuse_ideal(ik) = 0;
    wuse_eff_ideal(ik) = 0;
    
    %% Generating 10000 trajectories with random entry and exist angles.
    for ij = 1 : 10000
        breakdown_HTCE(i,ik,ij)=0;
        breakdown_GHO(ik,ij)=0;
        breakdown_fixed(ik,ij)=0;
        theta_i(ik,ij)=2*pi*rand(1);
        theta_o(ik,ij)=2*pi*rand(1);
        theta(ik,ij)=theta_i(ik,ij)-theta_o(ik,ij);
        if theta(ik,ij) < 0
            theta(ik,ij) = abs(theta(ik,ij));
        end    
        L(ik,ij)=sqrt(2*R^2*(1-cos(theta(ik,ij))));
        Lh(ik,ij) = 0.5*L(ik,ij);
        L_OM(ik,ij) = sqrt(R^2-Lh(ik,ij)^2);
        
        %% Calculate the handover triggering distance of the ideal
        %% algorithm.
        r_ideal(ik,ij) = sqrt(L_OM(ik,ij)^2+(Lh(ik,ij)-v(ik)*tau_o)^2);
        noise_ideal(ik,ij) = getnoise(sigma(ik));
        RSS_ideal(ik,ij) = P_tx-PL_ref-10*beta*log10(r_ideal(ik,ij))+noise_ideal(ik,ij);
        r_ideal_est(ik,ij) = 10^((P_tx-PL_ref-RSS_ideal(ik,ij))/10/beta);
        m0=m0+1;
        if (theta(ik,ij)>=acos(1-v(ik)^2*(tau_i+tau_o)^2/2/R^2)) & (theta(ik,ij)<=2*pi-acos(1-v(ik)^2*(tau_i+tau_o)^2/2/R^2))
            if R < v(ik)*tau_o
                boundary_ideal(ik,ij) = R;
                wuse_ideal(ik) = wuse_ideal(ik)+L(ik,ij)/v(ik)-tau_i;
            else
        if r_ideal_est(ik,ij)>L_OM(ik,ij) & r_ideal_est(ik,ij)<R
        boundary_ideal(ik,ij) = Lh(ik,ij) - sqrt(r_ideal_est(ik,ij)^2-L_OM(ik,ij)^2);
        if boundary_ideal(ik,ij)>=tau_o*v(ik)
            wuse_eff_ideal(ik) = wuse_eff_ideal(ik)+L(ik,ij)/v(ik)-tau_i+tau_o- boundary_ideal(ik,ij)/v(ik);
            wuse_ideal(ik) = wuse_ideal(ik)+L(ik,ij)/v(ik)-tau_i+tau_o- boundary_ideal(ik,ij)/v(ik);
        end
        elseif r_ideal_est(ik,ij)>=R
            boundary_ideal(ik,ij) = 0;
            wuse_ideal(ik) = wuse_ideal(ik)+L(ik,ij)/v(ik)-tau_i;
        else
            boundary_ideal(ik,ij) = Lh(ik,ij);
            if (L(ik,ij)-tau_i*v(ik)) < v(ik)*tau_o  
                wuse_ideal(ik) = wuse_ideal(ik)+L(ik,ij)/v(ik)-tau_i;  
            else 
                 wuse_ideal(ik) =wuse_ideal(ik)+tau_o;
                 wuse_eff_ideal(ik) =wuse_eff_ideal(ik)+tau_o;
            end
        end
        distance_ideal(ik) = distance_ideal(ik)+boundary_ideal(ik,ij);
            end
        end
        wlantime(ik) = wlantime(ik) + L(ik,ij)/v(ik);         
        
        %% Calculate the probability of connection breakdowns, WLAN usage
        %% and handover triggering distance for HTCE.
        if (theta(ik,ij)>=acos(1-v(ik)^2*(tau_i+tau_o)^2/2/R^2)) & (theta(ik,ij)<=2*pi-acos(1-v(ik)^2*(tau_i+tau_o)^2/2/R^2))
            if R < v(ik)*tau_o
                num_break(i,ik,ij) = num_break(ik,ij)+1;
                wuse(i,ik) = wuse(i,ik)+L(ik,ij)/v(ik)-tau_i;
                L_bound_est(i,ik,ij) = R;
                L_bound_est_eff(i,ik,ij) = 0;
            else
                if v0(ik)<v_htce
                    r(ik) = R_hyst;
                else
                r(i,ik) = sqrt(v(ik)^2*tau_o^2+R^2-sqrt(2*(1-cos(pi-pi*Ph(i))))*v(ik)*tau_o*R);
                r(i,ik) = r(i,ik)-R*errorratio(ik);
                end
                L_bound(ik) = v(ik)*tau_o;
                l_OS1(ik,ij) = sqrt((v(ik).*0).^2+L_OM(ik,ij)^2);
                l_OS2(ik,ij) = sqrt((2*(v(ik)*0))^2+L_OM(ik,ij)^2);        
                noise_S1(ik,ij) = getnoise(sigma(ik));
                noise_S2(ik,ij) = getnoise(sigma(ik));
                RSS_S1(ik,ij) = P_tx-PL_ref-10*beta*log10(l_OS1(ik,ij))+noise_S1(ik,ij);
                RSS_S2(ik,ij) = P_tx-PL_ref-10*beta*log10(l_OS2(ik,ij))+noise_S2(ik,ij);
                l_OS1_est(ik,ij) = 10^((P_tx-PL_ref-RSS_S1(ik,ij))/10/beta);
                l_OS2_est(ik,ij) = 10^((P_tx-PL_ref-RSS_S2(ik,ij))/10/beta);
                noise_r(ik,ij) = getnoise(sigma(ik));
                RSS_r(i,ik,ij) = P_tx-PL_ref-10*beta*log10(r(i,ik))+noise_r(ik,ij);
                r_est(i,ik,ij) = 10^((P_tx-PL_ref-RSS_r(i,ik,ij))/10/beta);
                d_R2(ik,ij)=sqrt(0.5*R^2*(1-cos(theta(ik,ij))));
                        if r_est(i,ik,ij)>=R
                            breakdown_HTCE(i,ik,ij)=1;
                           num_break(i,ik) = num_break(i,ik)+1;
                           wuse(i,ik) = wuse(i,ik)+L(ik,ij)/v(ik)-tau_i;
                           L_bound_est(i,ik,ij) = 0;
                           L_bound_est_eff(i,ik,ij) = 0;
                        elseif r_est(i,ik,ij)<L_OM(ik,ij)
                               if (L(ik,ij)-tau_i*v(ik)) < v(ik)*tau_o                    
                               breakdown_HTCE(i,ik,ij)=1;
                               num_break(i,ik) = num_break(i,ik)+1;
                               wuse(i,ik) = wuse(i,ik)+L(ik,ij)/v(ik)-tau_i;   
                               L_bound_est(i,ik,ij) = L(ik,ij)-tau_i*v(ik);
                               L_bound_est_eff(i,ik,ij) = 0;
                               else                                 
                                   wuse(i,ik) =wuse(i,ik)+tau_o;
                                   wuse_eff(i,ik) =wuse_eff(i,ik)+0+tau_o;
                                   L_bound_est(i,ik,ij) = L(ik,ij)-v(ik)*tau_i;
                                   L_bound_est_eff(i,ik,ij) = L(ik,ij)-v(ik)*tau_i;
                                   n1=n1+1;
                               end                               
                        else
                            d_r(i,ik,ij)=sqrt(r_est(i,ik,ij)^2-L_OM(ik,ij)^2);
                            L_bound_est(i,ik,ij)=d_R2(ik,ij)-d_r(i,ik,ij);
                            if L_bound_est(i,ik,ij) ~= 0  &  L_bound_est(i,ik,ij) < v(ik)*tau_o
                                breakdown_HTCE(i,ik,ij)=1;
                                num_break(i,ik) = num_break(i,ik)+1;
                                wuse(i,ik) = wuse(i,ik)+L(ik,ij)/v(ik)-tau_i;
                                L_bound_est_eff(i,ik,ij) = 0;
                            elseif L_bound_est(i,ik,ij) ~= 0
                                wuse(i,ik) =wuse(i,ik)+L(ik,ij)/v(ik)-tau_i+tau_o-L_bound_est(i,ik,ij)/v(ik);
                                wuse_eff(i,ik) =wuse_eff(i,ik)+L(ik,ij)/v(ik)-tau_i+tau_o-L_bound_est(i,ik,ij)/v(ik);
                                L_bound_est_eff(i,ik,ij) = L_bound_est(i,ik,ij);
                                n1=n1+1;
                            end
                        end                                              
            end
            esterror(i,ik,ij) = abs(L_bound_est(i,ik,ij)-L_bound(ik))/L_bound(ik);
                        esterror_sum(i,ik) = esterror_sum(i,ik)+esterror(i,ik,ij); 
                        m = m+1;
                        distance_HTCE(i,ik) = distance_HTCE(i,ik)+L_bound_est(i,ik,ij);
                        distance_HTCE_eff(i,ik) = distance_HTCE_eff(i,ik)+L_bound_est_eff(i,ik,ij);
        end
        
        %% Calculate the probability of connection breakdowns and WLAN
        %% usage for the hysteresis based method.
            if (theta(ik,ij)>=acos(1-v(ik)^2*(tau_i+tau_o)^2/2/R^2)) & (theta(ik,ij)<=2*pi-acos(1-v(ik)^2*(tau_i+tau_o)^2/2/R^2)) 
            noise_rHY(ik,ij) = getnoise(sigma(ik));
            RSS_rHY(ik,ij) = P_tx-PL_ref-10*beta*log10(R_hyst)+noise_rHY(ik,ij);
            rHY_est(ik,ij) = 10^((P_tx-PL_ref-RSS_rHY(ik,ij))/10/beta);
            Min_err_sum(ik) = 0;
            Min_err(ik,ij) = abs(rHY_est(ik,ij)-R_hyst)/R_hyst;
            Min_err_sum(ik) = Min_err_sum(ik)+Min_err(ik,ij);
            m1 = m1+1;
            d_R2_Min(ik,ij)=sqrt(0.5*R^2*(1-cos(theta(ik,ij))));
            if rHY_est(ik,ij)>=R
                breakdown_GHO(ik,ij)=1;
                num_break_Min(ik) = num_break_Min(ik)+1;
                wuse_Min(ik) = wuse_Min(ik)+L(ik,ij)/v(ik)-tau_i;
                L_bound_est_GHO(ik,ij) = 0;
                L_bound_est_GHO_eff(ik,ij) = 0;
            elseif rHY_est(ik,ij)<=L_OM(ik,ij) & rHY_est(ik,ij) ~=0
                   if (L(ik,ij)-tau_i*v(ik)) < v(ik)*tau_o  
                   breakdown_GHO(ik,ij)=1;
                   num_break_Min(ik) = num_break_Min(ik)+1;
                   wuse_Min(ik) = wuse_Min(ik)+L(ik,ij)/v(ik)-tau_i;
                   L_bound_est_GHO(ik,ij) = L(ik,ij)-tau_i*v(ik);
                   L_bound_est_GHO_eff(ik,ij) = 0;
                   else wuse_Min(ik) = wuse_Min(ik)+0+tau_o;
                       wuse_eff_Min(ik) = wuse_eff_Min(ik)+0+tau_o;
                       L_bound_est_GHO(ik,ij) = L(ik,ij)-tau_i*v(ik);
                       L_bound_est_GHO_eff(ik,ij) = L_bound_est_GHO(ik,ij);
                       n2=n2+1;
                   end 
            else
            d_r_Min(ik,ij)=sqrt(rHY_est(ik,ij)^2-R^2+d_R2_Min(ik,ij)^2);
            d2_Min(ik,ij)=d_R2_Min(ik,ij)-d_r_Min(ik,ij);
            L_bound_est_GHO(ik,ij) = d2_Min(ik,ij);
            if d2_Min(ik,ij) ~= 0  &  d2_Min(ik,ij) < v(ik)*tau_o
                breakdown_GHO(ik,ij)=1;
                num_break_Min(ik) = num_break_Min(ik)+1;
                wuse_Min(ik) = wuse_Min(ik)+L(ik,ij)/v(ik)-tau_i;
                L_bound_est_GHO_eff(ik,ij) = 0;
            elseif d2_Min(ik,ij) ~= 0
                wuse_Min(ik) =wuse_Min(ik)+L(ik,ij)/v(ik)-tau_i+tau_o-d2_Min(ik,ij)/v(ik);
                wuse_eff_Min(ik) =wuse_eff_Min(ik)+L(ik,ij)/v(ik)-tau_i+tau_o-d2_Min(ik,ij)/v(ik);
                L_bound_est_GHO_eff(ik,ij) = L_bound_est_GHO(ik,ij);
                n2=n2+1;
            end
            end
            distance_GHO(ik) = distance_GHO(ik)+L_bound_est_GHO(ik,ij);
        distance_GHO_eff(ik) = distance_GHO_eff(ik)+L_bound_est_GHO_eff(ik,ij);
            end
    
        %% Calculate the probability of connection breakdowns and WLAN
        %% usage for the fixed RSS based method.
        if (theta(ik,ij)>=acos(1-v(ik)^2*(tau_i+tau_o)^2/2/R^2)) & (theta(ik,ij)<=2*pi-acos(1-v(ik)^2*(tau_i+tau_o)^2/2/R^2))
            noise_fixed(ik,ij) = getnoise(sigma(ik));
            RSS_fixed(ik,ij) = P_tx-PL_ref-10*beta*log10(R_fixed)+noise_fixed(ik,ij);
            R_fixed_est(ik,ij) = 10^((P_tx-PL_ref-RSS_fixed(ik,ij))/10/beta);
            fixed_err_sum(ik) = 0;
            fixed_err(ik,ij) = abs(R_fixed_est(ik,ij)-R_fixed)/R_fixed;
            fixed_err_sum(ik) = fixed_err_sum(ik)+fixed_err(ik,ij);
            m3 = m3+1;
            d_R2_fixed(ik,ij)=sqrt(0.5*R^2*(1-cos(theta(ik,ij))));
            if R_fixed_est(ik,ij)>=R
                breakdown_fixed(ik,ij)=1;
                num_break_fixed(ik) = num_break_fixed(ik)+1;
                wuse_fixed(ik) = wuse_fixed(ik)+L(ik,ij)/v(ik)-tau_i;
                L_bound_est_fixed(ik,ij) = 0;
                L_bound_est_fixed_eff(ik,ij) = 0;
            elseif R_fixed_est(ik,ij)<=L_OM(ik,ij) & R_fixed_est(ik,ij) ~=0
                   if (L(ik,ij)-tau_i*v(ik)) < v(ik)*tau_o  
                   breakdown_fixed(ik,ij)=1;
                   num_break_fixed(ik) = num_break_fixed(ik)+1;
                   wuse_fixed(ik) = wuse_fixed(ik)+L(ik,ij)/v(ik)-tau_i;
                   L_bound_est_fixed(ik,ij) = L(ik,ij)-tau_i*v(ik);
                   L_bound_est_fixed_eff(ik,ij) = 0;
                   else wuse_fixed(ik) = wuse_fixed(ik)+0+tau_o;
                       wuse_eff_fixed(ik) = wuse_eff_fixed(ik)+0+tau_o;
                       L_bound_est_fixed(ik,ij) = L(ik,ij)-tau_i*v(ik);
                       L_bound_est_fixed_eff(ik,ij) = L_bound_est_fixed(ik,ij);
                       n3=n3+1;
                   end 
            else
            d_r_fixed(ik,ij)=sqrt(R_fixed_est(ik,ij)^2-R^2+d_R2_fixed(ik,ij)^2);
            d2_fixed(ik,ij)=d_R2_fixed(ik,ij)-d_r_fixed(ik,ij);
            L_bound_est_fixed(ik,ij) = d2_fixed(ik,ij);
            if d2_fixed(ik,ij) ~= 0  &  d2_fixed(ik,ij) < v(ik)*tau_o
                breakdown_fixed(ik,ij)=1;
                num_break_fixed(ik) = num_break_fixed(ik)+1;
                wuse_fixed(ik) = wuse_fixed(ik)+L(ik,ij)/v(ik)-tau_i;
                L_bound_est_fixed_eff(ik,ij) = 0;
            elseif d2_fixed(ik,ij) ~= 0
                wuse_fixed(ik) =wuse_fixed(ik)+L(ik,ij)/v(ik)-tau_i+tau_o-d2_fixed(ik,ij)/v(ik);
                wuse_eff_fixed(ik) =wuse_eff_fixed(ik)+L(ik,ij)/v(ik)-tau_i+tau_o-d2_fixed(ik,ij)/v(ik);
                L_bound_est_fixed_eff(ik,ij) = L_bound_est_fixed(ik,ij);
                n3=n3+1;
            end
            end
            distance_fixed(ik) = distance_fixed(ik)+L_bound_est_fixed(ik,ij);
        distance_fixed_eff(ik) = distance_fixed_eff(ik)+L_bound_est_fixed_eff(ik,ij);
        end    
    end
    esterror_ave(i,ik) = esterror_sum(i,ik)/m;
    Pb(i,ik) = num_break(i,ik)/ij*100;
    Pb_Min(ik) = num_break_Min(ik)/ij*100;
    Pb_fixed(ik) = num_break_fixed(ik)/ij*100;
    wuse(i,ik) = wuse(i,ik)/wlantime(ik)*100;
    wuse_eff(i,ik) = wuse_eff(i,ik)/wlantime(ik)*100;
    wuse_eff_Min(ik) = wuse_eff_Min(ik)/wlantime(ik)*100;
    wuse_Min(ik) = wuse_Min(ik)/wlantime(ik)*100;
    wuse_eff_fixed(ik) = wuse_eff_fixed(ik)/wlantime(ik)*100;
    wuse_fixed(ik) = wuse_fixed(ik)/wlantime(ik)*100;
    wuse_eff_ideal(ik) = wuse_eff_ideal(ik)/wlantime(ik)*100;
    distance_HTCE_ave(i,ik) = distance_HTCE(i,ik)/m;
    distance_GHO_ave(ik) = distance_GHO(ik)/m1;
    distance_fixed_ave(ik) = distance_fixed(ik)/m3;
    distance_HTCE_eff_ave(i,ik) = distance_HTCE_eff(i,ik)/n1;
    distance_GHO_eff_ave(ik) = distance_GHO_eff(ik)/n2;
    distance_fixed_eff_ave(ik) = distance_fixed_eff(ik)/n3;
    distance_ideal_ave(ik) = distance_ideal(ik)/m;
end
end

%% Plot the figure for probability of connection breakdowns.
plot(v0,Pb_fixed,'-y.');
hold on
plot(v0,Pb_Min,'-r.');
hold on
plot(v0,Pb(1,1:length(v0)),'-b+');
hold on
plot(v0,Pb(2,1:length(v0)),'-gx');
hold on
plot(v0,Pb(3,1:length(v0)),'-kd');
hold on
plot(v0,Pb(4,1:length(v0)),'-mo');
hold on
legend('Fixed RSS based','Hysteresis based','HTTE P_b=0.02','HTTE P_b=0.4','HTTE P_b=0.6','HTTE P_b=0.8');
grid  ('on')
xlabel('Speed (km/h)')
ylabel('Connection Breakdown Percentage')

%% Plot the figure for the total WLAN usage.
figure
plot(v0,wuse_fixed,'-y.');
hold on
plot(v0,wuse_Min,'-r.');
hold on
plot(v0,wuse(1,1:length(v0)),'-b+');
hold on
plot(v0,wuse(2,1:length(v0)),'-gx');
hold on
plot(v0,wuse(3,1:length(v0)),'-kd');
hold on
plot(v0,wuse(4,1:length(v0)),'-mo');
hold on
legend('Fixed RSS based','Hysteresis based','HTCE P_b=0.02','HTCE P_b=0.4','HTCE P_b=0.6','HTCE P_b=0.8');
grid  ('on')
xlabel('Speed (km/h)')
ylabel('Total WLAN Usage')

%% Plot the figure for the handover triggering distance.
figure
plot(v0,distance_ideal_ave,'-r*');
hold on
plot(v0,distance_fixed_ave,'-y.');
hold on
plot(v0,distance_GHO_ave,'-r.');
hold on
plot(v0,distance_HTCE_ave(1,:),'-b+');
hold on
plot(v0,distance_HTCE_ave(2,:),'-gx');
hold on
plot(v0,distance_HTCE_ave(3,:),'-kd');
hold on
plot(v0,distance_HTCE_ave(4,:),'-mo');
hold on
legend('Ideal','Fixed RSS based','Hysteresis based','HTCE P_b=0.02','HTCE P_b=0.4','HTCE P_b=0.6','HTCE P_b=0.8');
grid  ('on')
xlabel('Speed (km/h)')
ylabel('Handover Trigger Distance (m)')

%% Plot the figure for the breakdown-free WLAN usage.
figure
plot(v0,wuse_eff_fixed,'-y.');
hold on
plot(v0,wuse_eff_Min,'-r.');
hold on
plot(v0,wuse_eff(1,1:length(v0)),'-b+');
hold on
plot(v0,wuse_eff(2,1:length(v0)),'-gx');
hold on
plot(v0,wuse_eff(3,1:length(v0)),'-kd');
hold on
plot(v0,wuse_eff(4,1:length(v0)),'-mo');
hold on
legend('Fixed RSS based','Hysteresis based','HTCE P_b=0.02','HTCE P_b=0.4','HTCE P_b=0.6','HTCE P_b=0.8');
grid  ('on')
xlabel('Speed (km/h)')
ylabel('Breakdown-free WLAN Usage')
